@@ -1,4 +1,3 @@
-import os
 import sys
 import time
 import threading
@@ -22,35 +21,35 @@ def data_collector(iterable, def_buf_size=5242880):
         Returns:
             A generator object
     '''
-    buffer = ''
+    buf = ''
     for data in iterable:
-        buffer += data
-        if len(buffer) >= def_buf_size:
-            output = buffer[:def_buf_size]
-            buffer = buffer[def_buf_size:]
+        buf += data
+        if len(buf) >= def_buf_size:
+            output = buf[:def_buf_size]
+            buf = buf[def_buf_size:]
             yield output
-    if len(buffer) > 0:
-        yield buffer
+    if len(buf) > 0:
+        yield buf
 
 def upload_part(upload_func, progress_cb, part_no, part_data):
-        num_retries = 5
-        def _upload_part(retries_left=num_retries):
-            try:
-                with contextlib.closing(StringIO.StringIO(part_data)) as f:
-                    f.seek(0)
-                    cb = lambda c,t:progress_cb(part_no, c, t) if progress_cb else None
-                    upload_func(f, part_no, cb=cb, num_cb=100)
-            except Exception, exc:
-                retries_left -= 1
-                if retries_left > 0:
-                    return _upload_part(retries_left=retries_left)
-                else:
-                    return threading.ThreadError(repr(threading.current_thread()) + ' ' + repr(exc))
-        return _upload_part()
+    num_retries = 5
+    def _upload_part(retries_left=num_retries):
+        try:
+            with contextlib.closing(StringIO.StringIO(part_data)) as f:
+                f.seek(0)
+                cb = lambda c,t:progress_cb(part_no, c, t) if progress_cb else None
+                upload_func(f, part_no, cb=cb, num_cb=100)
+        except Exception, exc:
+            retries_left -= 1
+            if retries_left > 0:
+                return _upload_part(retries_left=retries_left)
+            else:
+                return threading.ThreadError(repr(threading.current_thread()) + ' ' + repr(exc))
+    return _upload_part()
 
 def upload(bucket, aws_access_key, aws_secret_key,
            iterable, key, progress_cb=None,
-           parallelism=5, replace=False, secure=True,
+           threads=5, replace=False, secure=True,
            connection=None):
     ''' Upload data to s3 using the s3 multipart upload API.
 
@@ -64,7 +63,7 @@ def upload(bucket, aws_access_key, aws_secret_key,
             key: the name of the key to create in the s3 bucket
             progress_cb: will be called with (part_no, uploaded, total)
             each time a progress update is available.
-            parallelism: the number of threads to use while uploading.
+            threads: the number of threads to use while uploading.
             replace: will replace the key in s3 if set to true. (Default is false)
             secure: use ssl when talking to s3. (Default is true)
             connection: used for testing
@@ -84,7 +83,7 @@ def upload(bucket, aws_access_key, aws_secret_key,
     upload.counter = 0
 
     try:
-        tpool = pool.ThreadPool(processes=parallelism)
+        tpool = pool.ThreadPool(processes=threads)
 
         def check_errors():
             try:
@@ -95,7 +94,7 @@ def upload(bucket, aws_access_key, aws_secret_key,
                 raise exc
 
         def waiter():
-            while upload.counter >= parallelism:
+            while upload.counter >= threads:
                 check_errors()
                 time.sleep(0.1)
 
@@ -122,3 +121,44 @@ def upload(bucket, aws_access_key, aws_secret_key,
         multipart_obj.cancel_upload()
         tpool.terminate()
         raise
+
+def cli():
+    from optparse import OptionParser
+    parser = OptionParser()
+    parser.add_option('-b', '--bucket', dest='bucket',
+                      help='the s3 bucket to upload to')
+    parser.add_option('-k', '--key', dest='key',
+                      help='the name of the key to create in the bucket')
+    parser.add_option('-K', '--aws_key', dest='aws_key',
+                      help='aws access key')
+    parser.add_option('-s', '--aws_secret', dest='aws_secret',
+                      help='aws secret key')
+    parser.add_option('-d', '--data', dest='data',
+                      help='the data to upload to s3 -- if left blank will be read from STDIN')
+    parser.add_option('-t', '--threads', dest='threads', default=5, type='int',
+                      help='number of threads to use while uploading in parallel')
+    (options, args) = parser.parse_args()
+
+    if len(sys.argv) < 2:
+        parser.print_help()
+        sys.exit(0)
+
+    if not options.bucket:
+        parser.error('bucket not provided')
+    if not options.key:
+        parser.error('key not provided')
+    if not options.aws_key:
+        parser.error('aws access key not provided')
+    if not options.aws_secret:
+        parser.error('aws secret key not provided')
+
+    data = sys.stdin if not options.data else [options.data]
+
+    def cb(part_no, uploaded, total):
+        print part_no, uploaded, total
+
+    upload(options.bucket, options.aws_key, options.aws_secret, data_collector(data), options.key,
+           progress_cb=cb, replace=True, threads=options.threads)
+
+if __name__ == '__main__':
+    cli()
